@@ -15,7 +15,7 @@ use signals::Signals;
 use std::ffi::{CStr, CString};
 use std::fs::File;
 use std::io;
-use std::io::{Error, ErrorKind};
+use std::io::Error;
 use std::mem::MaybeUninit;
 use std::ops::Deref;
 use std::os::fd::OwnedFd;
@@ -428,7 +428,7 @@ pub fn create_pty_with_spawn(
     };
 
     if res < 0 {
-        return Err(Error::new(ErrorKind::Other, "openpty failed"));
+        return Err(Error::other("openpty failed"));
     }
 
     let mut shell_program = shell;
@@ -453,13 +453,20 @@ pub fn create_pty_with_spawn(
         cmd
     };
 
-    #[cfg(not(any(target_os = "macos", target_os = "freebsd")))]
+    #[cfg(target_os = "linux")]
     {
         // If running inside a flatpak sandbox.
         // Must retrieve $SHELL from outside the sandbox, so ask the host.
         if std::path::PathBuf::from("/.flatpak-info").exists() {
             builder = Command::new("flatpak-spawn");
-            let mut with_args = vec!["--host".to_string(), "--watch-bus".to_string()];
+
+            let mut with_args = vec![
+                "--host".to_string(),
+                "--watch-bus".to_string(),
+                "--env=COLORTERM=truecolor".to_string(),
+                "--env=TERM=rio".to_string(),
+            ];
+
             if let Some(directory) = working_directory {
                 with_args.push(format!(
                     "--directory={}",
@@ -467,15 +474,6 @@ pub fn create_pty_with_spawn(
                 ));
             }
 
-            // Map only base environment variables
-            for (k, v) in std::env::vars_os() {
-                let key: String = k.into_string().unwrap_or_default();
-                let value: String = v.into_string().unwrap_or_default();
-                with_args.push(format!("--env={key}={value}"));
-            }
-
-            with_args.push("--env=TERM_PROGRAM=rio".to_string());
-
             let output = std::process::Command::new("flatpak-spawn")
                 .args(["--host", "sh", "-c", "echo $SHELL"])
                 .output()?;
@@ -508,7 +506,7 @@ pub fn create_pty_with_spawn(
             // Create a new process group.
             let err = libc::setsid();
             if err == -1 {
-                return Err(Error::new(ErrorKind::Other, "Failed to set session id"));
+                return Err(Error::other("Failed to set session id"));
             }
 
             if is_controling_terminal {
@@ -618,10 +616,9 @@ pub fn create_pty_with_fork(shell: &str, columns: u16, rows: u16) -> Result<Pty,
     } {
         0 => {
             default_shell_command(shell_program);
-            Err(Error::new(
-                ErrorKind::Other,
-                format!("forkpty has reach unreachable with {}", shell_program),
-            ))
+            Err(Error::other(format!(
+                "forkpty has reach unreachable with {shell_program}"
+            )))
         }
         id if id > 0 => {
             // TODO: Currently we fork the process and don't wait to know if led to failure
@@ -649,10 +646,9 @@ pub fn create_pty_with_fork(shell: &str, columns: u16, rows: u16) -> Result<Pty,
                 signals_token: corcovado::Token(0),
             })
         }
-        _ => Err(Error::new(
-            ErrorKind::Other,
-            format!("forkpty failed using {}", shell_program),
-        )),
+        _ => Err(Error::other(format!(
+            "forkpty failed using {shell_program}"
+        ))),
     }
 }
 
@@ -836,11 +832,11 @@ fn get_pw_entry(buf: &mut [i8; 1024]) -> Result<Passwd<'_>, Error> {
     let entry = unsafe { entry.assume_init() };
 
     if status < 0 {
-        return Err(Error::new(ErrorKind::Other, "getpwuid_r failed"));
+        return Err(Error::other("getpwuid_r failed"));
     }
 
     if res.is_null() {
-        return Err(Error::new(ErrorKind::Other, "pw not found"));
+        return Err(Error::other("pw not found"));
     }
 
     // Sanity check.
